@@ -29,6 +29,7 @@
 #include "data_structure/privileges.h"
 #include "data_structure/system_prefs.h"
 #include "utils.h"
+#include "mode_data/calcinprogress.h"
 #include "bakery/View/View.h"
 #include <bakery/Utilities/BusyCursor.h>
 
@@ -105,6 +106,109 @@ protected:
 
   Glib::ustring get_find_where_clause_quick(const Glib::ustring& table_name, const Gnome::Gda::Value& quick_search) const;
 
+
+  virtual void set_entered_field_data(const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value&  value);
+  virtual void set_entered_field_data(const Gtk::TreeModel::iterator& row, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value);
+
+
+  class FieldInRecord
+  {
+  public:
+    FieldInRecord()
+    {}
+
+    FieldInRecord(const Glib::ustring& table_name, const sharedptr<const Field>& field, const sharedptr<const Field>& key, const Gnome::Gda::Value& key_value)
+    : m_table_name(table_name), m_field(field), m_key(key), m_key_value(key_value)
+    {
+    }
+
+    FieldInRecord(const sharedptr<const LayoutItem_Field>& layout_item, const Glib::ustring& parent_table_name, const sharedptr<const Field>& parent_key, const Gnome::Gda::Value& key_value, const Document_Glom& document)
+    : m_key_value(key_value)
+    {
+      m_field = layout_item->get_full_field_details();
+
+      if(layout_item->get_has_relationship_name())
+      {
+        //The field is in a related table.
+        sharedptr<const Relationship> rel = layout_item->get_relationship();
+        if(rel)
+        {
+          m_table_name = rel->get_to_table();
+
+          //Actually a foreign key:
+          m_key = document.get_field(m_table_name, rel->get_to_field());
+        }
+      }
+      else
+      {
+        m_table_name = parent_table_name;
+        m_key = parent_key;
+      }
+    }
+
+    //Identify the field:
+    Glib::ustring m_table_name;
+    sharedptr<const Field> m_field;
+
+    //Identify the record:
+    sharedptr<const Field> m_key;
+    Gnome::Gda::Value m_key_value;
+  };
+
+  /** Calculate values for fields, set them in the database, and show them in the layout.
+   * @param field_changed The field that has changed, causing other fields to be recalculated because they use its value.
+   * @param primary_key The primary key field for this table.
+   * @param priamry_key_value: The primary key value for this record.
+   * @param first_calc_field: false if this is called recursively.
+   */
+  void do_calculations(const FieldInRecord& field_changed, bool first_calc_field);
+
+  typedef std::map<Glib::ustring, CalcInProgress> type_field_calcs;
+
+  /** Get the fields whose values should be recalculated when @a field_name changes.
+   */
+  type_field_calcs get_calculated_fields(const Glib::ustring& table_name, const Glib::ustring& field_name);
+
+  typedef std::list< sharedptr<LayoutItem_Field> > type_list_field_items;
+
+  /** Get the fields used, if any, in the calculation of this field.
+   */
+  type_list_field_items get_calculation_fields(const Glib::ustring& table_name, const sharedptr<const Field>& field);
+
+  void calculate_field(const FieldInRecord& field_in_record);
+
+  void calculate_field_in_all_records(const Glib::ustring& table_name, const sharedptr<const Field>& field);
+  void calculate_field_in_all_records(const Glib::ustring& table_name, const sharedptr<const Field>& field, const sharedptr<const Field>& primary_key);
+
+  typedef std::map<Glib::ustring, Gnome::Gda::Value> type_map_fields;
+  //TODO: Performance: This is massively inefficient:
+  type_map_fields get_record_field_values(const Glib::ustring& table_name, const sharedptr<const Field> primary_key, const Gnome::Gda::Value& primary_key_value);
+
+
+  void do_lookups(const FieldInRecord& field_in_record, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& field_value);
+
+  typedef std::pair< sharedptr<LayoutItem_Field>, sharedptr<Relationship> > type_pairFieldTrigger;
+  typedef std::list<type_pairFieldTrigger> type_list_lookups;
+
+  /** Get the fields whose values should be looked up when @a field_name changes, with
+   * the relationship used to lookup the value.
+   */
+  type_list_lookups get_lookup_fields(const Glib::ustring& table_name, const Glib::ustring& field_name) const;
+
+
+  /** Get the value of the @a source_field from the @a relationship, using the @a key_value.
+   */
+  Gnome::Gda::Value get_lookup_value(const Glib::ustring& table_name, const sharedptr<const Relationship>& relationship, const sharedptr<const Field>& source_field, const Gnome::Gda::Value & key_value);
+
+
+  virtual void refresh_related_fields(const FieldInRecord& field_in_record_changed, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& field_value);
+
+
+
+  bool set_field_value_in_database(const FieldInRecord& field_in_record, const Gnome::Gda::Value& field_value, bool use_current_calculations = false);
+  bool set_field_value_in_database(const FieldInRecord& field_in_record, const Gtk::TreeModel::iterator& row, const Gnome::Gda::Value& field_value, bool use_current_calculations = false);
+
+
   type_vecStrings get_database_groups() const;
   type_vecStrings get_database_users(const Glib::ustring& group_name = Glib::ustring()) const;
   Privileges get_table_privileges(const Glib::ustring& group_name, const Glib::ustring& table_name) const;
@@ -165,6 +269,7 @@ protected:
   void handle_error(const std::exception& ex) const; //TODO_port: This is probably useless now.
   bool handle_error() const;
 
+  type_field_calcs m_FieldsCalculationInProgress; //Prevent circular calculations and recalculations.
 };
 
 #endif //BASE_DB_H
