@@ -295,13 +295,19 @@ Gnome::Gda::Value GlomConversions::parse_value(Field::glom_field_type glom_type,
     gda_date.year = the_c_time.tm_year + 1900; //The C time starts at 1900.
     gda_date.month = the_c_time.tm_mon + 1; //The C month starts at 0.
     gda_date.day = the_c_time.tm_mday; //THe C mday starts at 1.
-    
+
     return Gnome::Gda::Value(gda_date);
   }
   else if(glom_type == Field::TYPE_TIME)
   {
     tm the_c_time = parse_time(text, the_locale, success);
 
+    if(!success)
+    {
+      //Fall back to trying both the current locale and C locale:
+      the_c_time = parse_time(text, success);
+    }
+
     Gnome::Gda::Time gda_time = {0, 0, 0, 0};
     gda_time.hour = the_c_time.tm_hour;
     gda_time.minute = the_c_time.tm_min;
@@ -341,7 +347,7 @@ Gnome::Gda::Value GlomConversions::parse_value(Field::glom_field_type glom_type,
     std::stringstream clocale_stream;
     clocale_stream.imbue( std::locale::classic() ); //The C locale.
     clocale_stream << the_number;
-    Glib::ustring  number_canonical_text = clocale_stream.str(); //Avoid << because it does implicit character conversion (though that might not be a problem here. Not sure). murrayc
+    Glib::ustring number_canonical_text = clocale_stream.str(); //Avoid << because it does implicit character conversion (though that might not be a problem here. Not sure). murrayc
 
     //TODO: What about the precision and width values?
     /* From the postgres docs:
@@ -387,7 +393,7 @@ tm GlomConversions::parse_date(const Glib::ustring& text, const std::locale& loc
    * When not, I get just zeros.
    */
   tm the_c_time = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-  
+
   std::ios_base::iostate err = std::ios_base::goodbit;  //The initialization is essential because time_get seems to a) not initialize this output argument and b) check its value.
 
   //For some reason, the stream must be instantiated after we get the facet. This is a worryingly strange "bug".
@@ -396,7 +402,7 @@ tm GlomConversions::parse_date(const Glib::ustring& text, const std::locale& loc
   the_stream << text;
 
   // Get a time_get facet:
- 
+
   typedef std::time_get<char> type_time_get;
   typedef type_time_get::iter_type type_iterator;
 
@@ -420,13 +426,13 @@ tm GlomConversions::parse_date(const Glib::ustring& text, const std::locale& loc
     {
       tm null_c_time = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
       the_c_time = null_c_time;
-      
+
       if(date.get_year() != Glib::Date::BAD_YEAR)
         the_c_time.tm_year = date.get_year() - 1900; //C years start are the AD year - 1900. So, 01 is 1901.
-      
+
       if(date.get_month() != Glib::Date::BAD_MONTH)
         the_c_time.tm_mon = date.get_month() - 1; //C months start at 0.
-        
+
       if(date.get_day() != Glib::Date::BAD_DAY)
         the_c_time.tm_mday = date.get_day(); //starts at 1
 
@@ -444,11 +450,13 @@ tm GlomConversions::parse_date(const Glib::ustring& text, const std::locale& loc
   }
 
   //Prevent some nonsense values:
+  //tm_day starts from 1.
   if(the_c_time.tm_mday == 0)
     the_c_time.tm_mday = 1;
 
-  if(the_c_time.tm_mon == 0)
-      the_c_time.tm_mon = 1;
+  //tm_mon starts from 0, so 0 is an acceptable value.
+  //if(the_c_time.tm_mon == 0)
+  //    the_c_time.tm_mon = 1;
 
   return the_c_time;
 }
@@ -461,7 +469,9 @@ tm GlomConversions::parse_time(const Glib::ustring& text, bool& success)
   //time_get() does not seem to work with non-C locales.  TODO: Try again.
   tm the_time = parse_time( text, std::locale("") /* the user's current locale */, success );
   if(success)
+  {
     return the_time;
+  }
   else
   {
     //Fallback:
@@ -515,6 +525,8 @@ tm GlomConversions::parse_time(const Glib::ustring& text, const std::locale& loc
   }
   else
   {
+    //std::cout << "  tg.get_time() failed" << text << std::endl;
+
     tm blank_time = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
     success = false;
     return blank_time;
@@ -566,6 +578,37 @@ bool GlomConversions::value_is_empty(const Gnome::Gda::Value& value)
   }
 }
 
+Gnome::Gda::Value GlomConversions::get_empty_value_suitable_for_python(Field::glom_field_type field_type)
+{
+  Gnome::Gda::Value value = get_empty_value(field_type);
+  if(value.get_value_type() == Gnome::Gda::VALUE_TYPE_NULL)
+  {
+    switch(field_type)
+    {
+      case(Field::TYPE_TEXT):
+        return Gnome::Gda::Value( Glib::ustring() );
+      case(Field::TYPE_NUMERIC):
+      {
+        return parse_value(0);
+      }
+      case(Field::TYPE_DATE):
+      {
+        bool success = false;
+        return parse_value(Field::TYPE_DATE, "01/01/00" /* arbitrary*/, success, true /* iso_format */);
+      }
+      case(Field::TYPE_TIME):
+      {
+        bool success = false;
+        return parse_value(Field::TYPE_TIME, "00:00" /* arbitrary*/, success, true /* iso_format */);
+      }
+      default:
+        return Gnome::Gda::Value(); //A NULL instance. Should never happen.
+    }
+  }
+  else
+    return value;
+}
+
 Gnome::Gda::Value GlomConversions::get_empty_value(Field::glom_field_type field_type)
 {
   switch(field_type)
@@ -810,6 +853,19 @@ Gnome::Gda::Value GlomConversions::parse_escaped_binary_data(const Glib::ustring
   return result;
 }
 
+Gnome::Gda::Value GlomConversions::convert_value(const Gnome::Gda::Value& value, Field::glom_field_type target_glom_type)
+{
+  const Field::glom_field_type source_glom_type = Field::get_glom_type_for_gda_type(value.get_value_type());
+  if(source_glom_type == target_glom_type)
+    return value; //No conversion necessary.
+  else
+  {
+    const Glib::ustring text = get_text_for_gda_value(source_glom_type, value);
+    bool test = false;
+    return parse_value(target_glom_type, text, test, true /* iso_format */);
+  }
 
+  return value;
+}
 
 
