@@ -27,6 +27,7 @@
 #include "../data_structure/field.h"
 #include "../data_structure/layout/layoutgroup.h"
 #include "../data_structure/layout/layoutitem_portal.h"
+#include "../data_structure/layout/layoutitem_button.h"
 #include "../data_structure/tableinfo.h"
 #include "../data_structure/groupinfo.h"
 #include "../data_structure/report.h"
@@ -95,7 +96,7 @@ public:
   Glib::ustring get_translation_original_locale() const;
 
   typedef std::vector< sharedptr<Relationship> > type_vecRelationships;
-  virtual type_vecRelationships get_relationships(const Glib::ustring& table_name) const;
+  virtual type_vecRelationships get_relationships(const Glib::ustring& table_name, bool plus_system_prefs = false) const;
   void set_relationships(const Glib::ustring& table_name, const type_vecRelationships& vecRelationships);
 
   sharedptr<Relationship> get_relationship(const Glib::ustring& table_name, const Glib::ustring& relationship_name) const;
@@ -120,6 +121,11 @@ public:
 
   virtual sharedptr<Field> get_field(const Glib::ustring& table_name, const Glib::ustring& strFieldName) const;
 
+  /** Use this after removing a field from a table,
+   * so that it is not used anymore in relationships, layouts, reports, etc.
+   */
+  void remove_field(const Glib::ustring& table_name, const Glib::ustring& field_name);
+
 
   typedef std::map<guint, sharedptr<LayoutGroup> > type_mapLayoutGroupSequence;
   type_mapLayoutGroupSequence get_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name) const;
@@ -146,7 +152,7 @@ public:
   virtual void change_field_name(const Glib::ustring& table_name, const Glib::ustring& strFieldNameOld, const Glib::ustring& strFieldNameNew);
 
   ///When a table name is changed, change it in the relationships and tables data:
-  virtual void change_table_name(const Glib::ustring& strTableNameOld, const Glib::ustring& strTableNameNew);
+  virtual void change_table_name(const Glib::ustring& table_name_old, const Glib::ustring& table_name_new);
 
   ///When a relationship name is changed, change it in layouts and reports:
   virtual void change_relationship_name(const Glib::ustring& table_name, const Glib::ustring& name, const Glib::ustring& name_new);
@@ -234,6 +240,8 @@ public:
 
   virtual void emit_userlevel_changed();
 
+  static sharedptr<TableInfo> Document_Glom::create_table_system_preferences(type_vecFields& fields);
+
 protected:
 
   //Overrides:
@@ -268,6 +276,8 @@ protected:
   static void set_node_attribute_value_as_value(xmlpp::Element* node, const Glib::ustring& strAttributeName, const Gnome::Gda::Value& value, Field::glom_field_type field_type);
   static Gnome::Gda::Value get_node_attribute_value_as_value(const xmlpp::Element* node, const Glib::ustring& strAttributeName, Field::glom_field_type field_type);
 
+  static sharedptr<Relationship> create_relationship_system_preferences(const Glib::ustring& table_name);
+
   AppState m_app_state;
   type_signal_userlevel_changed m_signal_userlevel_changed;
 
