@@ -46,6 +46,8 @@ void Box_DB_Table_Definition::init()
 
   pack_start(m_AddDel);
   m_colName = m_AddDel.add_column(_("Name"));
+  m_AddDel.prevent_duplicates(m_colName); //Don't allow adding of fields that already exist.
+  m_AddDel.set_prevent_duplicates_warning(_("This field already exists. Please choose a different field name"));
 
   m_colTitle = m_AddDel.add_column(_("Title"));
 
@@ -342,8 +344,27 @@ void Box_DB_Table_Definition::on_Properties_apply()
     }
     else
     {
-      change_definition(m_Field_BeingEdited, field_New);
-      m_Field_BeingEdited = field_New;
+      //Warn about recalculation:
+      bool change_def = true;
+      if(field_New->get_has_calculation())
+      {
+        if(field_New->get_calculation() != m_Field_BeingEdited->get_calculation())
+        {
+          //TODO: Only show this when there are > 100 records?
+          Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Recalculation Required")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE);
+          dialog.set_secondary_text(_("You have changed the calculation used by this field so Glom must recalculate the value in all records. If the table contains many records then this could take a long time."));
+          //dialog.set_transient_for(*get_app_window());
+          dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+          dialog.add_button(_("Recalculate"), Gtk::RESPONSE_OK);
+          change_def = (dialog.run() == Gtk::RESPONSE_OK);
+        }
+      }
+
+      if(change_def)
+      {
+        change_definition(m_Field_BeingEdited, field_New);
+        m_Field_BeingEdited = field_New;
+      }
     }
 
     //Update the list:
@@ -395,6 +416,14 @@ void Box_DB_Table_Definition::change_definition(const sharedptr<const Field>& fi
     {
       pDoc->change_field_name(m_table_name, field_name_old, field->get_name());
     }
+
+    //Recalculate if necessary:
+    if(field->get_has_calculation())
+    {
+      const Glib::ustring calculation = field->get_calculation();
+      if(calculation != fieldOld->get_calculation())
+        calculate_field_in_all_records(m_table_name, field);
+    }
   }
 
 
@@ -463,7 +492,7 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const
       //    COMMIT;
 
       Glib::RefPtr<Gnome::Gda::Transaction> transaction = Gnome::Gda::Transaction::create("glom_transaction_change_field_type");
-      bool test = gda_connection->begin_transaction(transaction);
+      const bool test = gda_connection->begin_transaction(transaction);
       if(test)
       {
         //TODO: Warn about a delay, and possible loss of precision, before actually doing this.
@@ -486,31 +515,71 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const
           WHERE _aaa <> '     ';  
           */
           Glib::ustring conversion_command;
+          const Glib::ustring field_name_old_quoted = "\"" + field_old->get_name() + "\"";
+          const Field::glom_field_type old_field_type = field_old->get_glom_type();
           switch(field->get_glom_type())
           {
-            case Field::TYPE_BOOLEAN: //CAST does not work if the destination type is numeric.
+            case Field::TYPE_BOOLEAN: //CAST does not work if the destination type is boolean.
             {
-              conversion_command = "FALSE"; //TODO: Find a way to convert: "to_number( " + field_old->get_name() + ", '999999999.99' )";
+              if(old_field_type == Field::TYPE_NUMERIC)
+              {
+                conversion_command = "(CASE WHEN " + field_name_old_quoted + " >0 THEN true "
+                                           "WHEN " + field_name_old_quoted + " = 0 THEN false "
+                                           "WHEN " + field_name_old_quoted + " IS NULL THEN false END)";
+              }
+              else if(old_field_type == Field::TYPE_TEXT)
+                conversion_command = "(" + field_name_old_quoted + " !~~* \'false\')"; // !~~* means ! ILIKE.
+              else //Dates and Times:
+                conversion_command = "(" + field_name_old_quoted + " IS NOT NULL')";
+
               break;
             }
             case Field::TYPE_NUMERIC: //CAST does not work if the destination type is numeric.
             {
-              conversion_command = "to_number( \"" + field_old->get_name() + "\", '999999999.99' )";
+              if(old_field_type == Field::TYPE_BOOLEAN)
+              {
+                conversion_command = "(CASE WHEN " + field_name_old_quoted + " = true THEN 1 "
+                                           "WHEN " + field_name_old_quoted + " = false THEN 0 "
+                                           "WHEN " + field_name_old_quoted + " IS NULL THEN 0 END)";
+              }
+              else
+              {
+                //We use to_number, with textcat() so that to_number always has usable data.
+                //Otherwise, it says 
+                //invalid input syntax for type numeric: " "
+                //
+                //We must use single quotes with the 0, otherwise it says "column 0 does not exist.".
+                conversion_command = "to_number( textcat(\'0\', " + field_name_old_quoted + "), '999999999.99999999' )";
+              }
+
               break;
             }
             case Field::TYPE_DATE: //CAST does not work if the destination type is numeric.
             {
-              conversion_command = "to_date( \"" + field_old->get_name() + "\", 'YYYYMMDD' )"; //TODO: standardise date storage format.
+              conversion_command = "to_date( " + field_name_old_quoted + ", 'YYYYMMDD' )"; //TODO: standardise date storage format.
               break;
             }
             case Field::TYPE_TIME: //CAST does not work if the destination type is numeric.
             {
-              conversion_command = "to_timestamp( \"" + field_old->get_name() + "\", 'HHMMSS' )";  //TODO: standardise time storage format.
+              conversion_command = "to_timestamp( " + field_name_old_quoted + ", 'HHMMSS' )";  //TODO: standardise time storage format.
               break;
             }
             default:
             {
-              conversion_command = "CAST(\"" +  field_old->get_name() + "\" AS " + field->get_sql_type() + ")";
+              //To Text:
+
+              //bool to text:
+              if(old_field_type == Field::TYPE_BOOLEAN)
+              {
+                 conversion_command = "(CASE WHEN " + field_name_old_quoted + " = true THEN \'true\' "
+                                            "WHEN " + field_name_old_quoted + " = false THEN \'false\' "
+                                            "WHEN " + field_name_old_quoted + " IS NULL THEN \'false\' END)";
+              }
+              else
+              {
+                //This works for most to-text conversions:
+                conversion_command = "CAST(\"" +  field_old->get_name() + "\" AS " + field->get_sql_type() + ")";
+              }
               break;
             }
           }
