@@ -45,7 +45,7 @@ ComboGlom::ComboGlom()
   init();
 }
 
-ComboGlom::ComboGlom(const LayoutItem_Field& field_second)
+ComboGlom::ComboGlom(const sharedptr<LayoutItem_Field>& field_second)
 : ComboGlomChoicesBase(field_second)
 {
   setup_menu();
@@ -66,7 +66,7 @@ void ComboGlom::init()
     //pack_start(m_Columns.m_col_second);
 
     Gtk::CellRenderer* cell_second = Gtk::manage(new Gtk::CellRendererText);
-    cell_second->property_xalign() = 0.0;
+    cell_second->property_xalign() = 0.0f;
 
     //Use the renderer:
     pack_start(*cell_second);
@@ -91,8 +91,8 @@ void ComboGlom::check_for_change()
     //Validate the input:
     bool success = false;
 
-    const LayoutItem_Field* layout_item = dynamic_cast<const LayoutItem_Field*>(get_layout_item());
-    Gnome::Gda::Value value = GlomConversions::parse_value(layout_item->m_field.get_glom_type(), new_text, layout_item->get_formatting_used().m_numeric_format, success);
+    sharedptr<const LayoutItem_Field> layout_item = sharedptr<const LayoutItem_Field>::cast_dynamic(get_layout_item());
+    const Gnome::Gda::Value value = GlomConversions::parse_value(layout_item->get_glom_type(), new_text, layout_item->get_formatting_used().m_numeric_format, success);
 
     if(success)
     {
@@ -103,7 +103,7 @@ void ComboGlom::check_for_change()
     else
     {
       //Tell the user and offer to revert or try again:
-      bool revert = glom_show_dialog_invalid_data(layout_item->m_field.get_glom_type());
+      bool revert = glom_show_dialog_invalid_data(layout_item->get_glom_type());
       if(revert)
       {
         set_text(m_old_text);
@@ -116,9 +116,9 @@ void ComboGlom::check_for_change()
 
 void ComboGlom::set_value(const Gnome::Gda::Value& value)
 {
-  const LayoutItem_Field* layout_item = dynamic_cast<const LayoutItem_Field*>(get_layout_item());
+  sharedptr<const LayoutItem_Field> layout_item = sharedptr<const LayoutItem_Field>::cast_dynamic(get_layout_item());
   if(layout_item)
-    set_text(GlomConversions::get_text_for_gda_value(layout_item->m_field.get_glom_type(), value, layout_item->get_formatting_used().m_numeric_format));
+    set_text(GlomConversions::get_text_for_gda_value(layout_item->get_glom_type(), value, layout_item->get_formatting_used().m_numeric_format));
 }
 
 void ComboGlom::set_text(const Glib::ustring& text)
@@ -144,11 +144,11 @@ void ComboGlom::set_text(const Glib::ustring& text)
 
 Gnome::Gda::Value ComboGlom::get_value() const
 {
-  const LayoutItem_Field* layout_item = dynamic_cast<const LayoutItem_Field*>(get_layout_item());
+  sharedptr<const LayoutItem_Field> layout_item = sharedptr<const LayoutItem_Field>::cast_dynamic(get_layout_item());
   bool success = false;
 
   const Glib::ustring text = get_text();
-  return GlomConversions::parse_value(layout_item->m_field.get_glom_type(), text, layout_item->get_formatting_used().m_numeric_format, success);
+  return GlomConversions::parse_value(layout_item->get_glom_type(), text, layout_item->get_formatting_used().m_numeric_format, success);
 }
 
 Glib::ustring ComboGlom::get_text() const
